///////////////////
// GDI interface //
///////////////////

#ifndef __PTC_IGDI_H
#define __PTC_IGDI_H

#include "isoft.h"
#include "surface.h"








class IGDI : public ISoftware
{
    public:

#ifndef __GDI__

        // dummy constructor
        IGDI(HWINDOW window=NULL) {};

#else

        // setup
        IGDI(HWINDOW window=NULL);
        virtual ~IGDI();

        // interface information
        virtual Interface::INFO GetInfo();
        virtual int GetModeList(List<MODE> &modelist);

        // display mode routines
        virtual int SetMode(MODE const &info);
        virtual int SetMode(int x,int y,int id,int layout,int output,int frequency);
        virtual int SetMode(int x,int y,FORMAT const &format,int layout,int output,int frequency);
        virtual MODE GetMode();

        // palette routines
        virtual int SetPalette(Palette &palette);
        virtual int GetPalette(Palette &palette);
        
        // hardware functions
        virtual int WaitForRetrace();
        virtual int SaveState();
        virtual int RestoreState();
        
        // primary surface operations
        virtual int SetPrimary(Surface &surface);
        virtual Surface* GetPrimary();
        virtual int SetOrigin(int x,int y);
        virtual int GetOrigin(int &x,int &y);

        // video memory management
        virtual int GetTotalVideoMemory();
        virtual int GetFreeVideoMemory();
        virtual int CompactVideoMemory();

        // console routines
        virtual int getch();
        virtual int kbhit();
        virtual int keydown(int key);

        // window routines
        virtual int SetTitle(char title[]);
        virtual int GetTitle(char title[]);

        // native access
        virtual int NativeType();
        virtual void* GetNative();

        // data access
        virtual void GetName(char name[]) const;
        virtual int GetXResolution() const;
        virtual int GetYResolution() const;
        virtual int GetBitsPerPixel() const;
        virtual int GetBytesPerPixel() const;
        virtual int GetLayout() const;
        virtual int GetOutput() const;
        virtual int GetFrequency() const;
        virtual FORMAT GetFormat() const;
        virtual HWINDOW GetWindow() const;

        // object state
        virtual int ok() const;

    protected:
        
        // surface update
        virtual int Update(Surface &src,void *extra);
        virtual int Update(Surface &src,RECTANGLE const &rect,void *extra);
        virtual int Update(Surface &src,RECTANGLE const &dest_rect,RECTANGLE const &src_rect,void *extra);

        // internal software surface
        class SURFACE : public Interface::SURFACE
        {
            public:
            
                // setup
                SURFACE();
                ~SURFACE();

                // initialize
                int Init(int &width,int &height,FORMAT &format,int &type,int &layout,int &advance);

                // surface interface
                virtual void* Lock(int wait);
                virtual void Unlock();
                virtual int LockCount();
                virtual int NativeType();
                virtual void* GetNative();

                // data
                void *Buffer;                   // memory buffer
                HBITMAP Bitmap;                 // bitmap handle
                BITMAPINFO *BitmapInfo;         // bitmap information
                int Count;                      // lock count
        };

        // internal surface management
        virtual Interface::SURFACE* RequestSurface(int &width,int &height,FORMAT &format,int &type,int &layout,int &advance);

        // object attach and detach
        virtual int Detach(Surface *surface);

    private:

        // window thread parm struct
        struct WINDOWTHREADINFO
        {
            IGDI *i;
            char *title;
            int width;
            int height;
        };

        // display routines
        int InitDisplay(int x,int y,int output);
        void CloseDisplay();

        // internal gdi routines
        FORMAT GetDeviceFormat(HDC dc);

        // window management
        int RegisterWindowClasses();
        int InitWindow(int width,int height,char *title=NULL);
        void CloseWindow();
        static DWORD WindowThread(WINDOWTHREADINFO *info);
        static LRESULT CALLBACK WndProc(HWND,UINT,WPARAM,LPARAM);

        // secondary management
        int InitSecondary();
        void CloseSecondary();

        // palette management
        int InitPalette();
        void ClosePalette();

        // interface lookup
        int RegisterWindow(HWND window);
        int UnregisterWindow(HWND window);
        static IGDI* LookupInterface(HWND window);

        // window criticial section manangement
        void InitWindowCriticalSection();
        void CloseWindowCriticalSection();


        // internals
        uint XResolution;                   // x resolution of display
        uint YResolution;                   // y resolution of display
        FORMAT Format;                      // pixel format of display
        int Output;                         // output mode
        Surface* Secondary;                 // secondary surface
        Surface* Recent;                    // most recently updated surface
        Palette InternalPalette;            // internal palette object
        LOGPALETTE *LogicalPalette;         // logical palette
        HPALETTE hPalette;                  // palette handle

        // window internals
        HWND Window;                        // output window handle
        HDC WindowDC;                       // output window device context
        int Managed;                        // managed window (0=false, 1=true)
        int getch_data;                     // static getch data (kludge)
        int kbhit_data;                     // static kbhit data (kludge)
        HANDLE hWindowThread;               // window thread
        HANDLE WindowEvent;                 // window signal event
        
        // status
        int Status;                         // status variable (1=OK, 0=INVALID)

        // static interface lookup list
        struct LOOKUP
        {
            HWND window;
            IGDI *i;
        };
        static List<LOOKUP> InterfaceLookupList;

        // window thread critical section
        static CRITICAL_SECTION WindowCriticalSection;
        static int WindowCriticalSectionReferenceCount;
#endif
};









#endif
