////////////////////////
// Software interface //
////////////////////////

#ifndef __PTC_ISOFTWARE_H
#define __PTC_ISOFTWARE_H

#include "iface.h"
#include "surface.h"







class ISoftware : public Interface
{
    public:

        // setup
        ISoftware(HWINDOW window=NULL);
        virtual ~ISoftware();

        // interface information
        virtual Interface::INFO GetInfo();
        virtual int GetModeList(List<MODE> &modelist);

        // display mode routines
        virtual int SetMode(MODE const &info);
        virtual int SetMode(int x,int y,int id,int layout,int output,int frequency);
        virtual int SetMode(int x,int y,FORMAT const &format,int layout,int output,int frequency);
        virtual MODE GetMode();

        // palette routines
        virtual int SetPalette(Palette &palette);
        virtual int GetPalette(Palette &palette);
        
        // hardware functions
        virtual int WaitForRetrace();
        virtual int SaveState();
        virtual int RestoreState();
        
        // primary surface operations
        virtual int SetPrimary(Surface &surface);
        virtual Surface* GetPrimary();
        virtual int SetOrigin(int x,int y);
        virtual int GetOrigin(int &x,int &y);

        // video memory management
        virtual int GetTotalVideoMemory();
        virtual int GetFreeVideoMemory();
        virtual int CompactVideoMemory();

        // console routines
        virtual int getch();
        virtual int kbhit();
        virtual int keydown(int key);

        // window routines
        virtual int SetTitle(char title[]);
        virtual int GetTitle(char title[]);

        // native access
        virtual int NativeType();
        virtual void* GetNative();

        // data access
        virtual void GetName(char name[]) const;
        virtual int GetXResolution() const;
        virtual int GetYResolution() const;
        virtual int GetBitsPerPixel() const;
        virtual int GetBytesPerPixel() const;
        virtual int GetLayout() const;
        virtual int GetOutput() const;
        virtual int GetFrequency() const;
        virtual FORMAT GetFormat() const;
        virtual HWINDOW GetWindow() const;

        // object state 
        virtual int ok() const;       

    protected:
        
        // surface clear
        virtual int Clear(Surface &surface,COLOR const &color);
        virtual int Clear(Surface &surface,RECTANGLE const &rect,COLOR const &color);

        // surface update
        virtual int Update(Surface &src,void *extra);
        virtual int Update(Surface &src,RECTANGLE const &rect,void *extra);
        virtual int Update(Surface &src,RECTANGLE const &dest_rect,RECTANGLE const &src_rect,void *extra);
        
        // surface bitblt
        virtual int BitBlt(Surface &src,Surface &dest,EFFECTS const *effects,void *extra);
        virtual int BitBlt(Surface &src,Surface &dest,RECTANGLE const &rect,EFFECTS const *effects,void *extra);
        virtual int BitBlt(Surface &src,RECTANGLE const &src_rect,
                           Surface &dest,RECTANGLE const &dest_rect,EFFECTS const *effects,void *extra);

        // surface stretchblt
        virtual int StretchBlt(Surface &src,Surface &dest,EFFECTS const *effects,void *extra);
        virtual int StretchBlt(Surface &src,Surface &dest,RECTANGLE const &rect,EFFECTS const *effects,void *extra);
        virtual int StretchBlt(Surface &src,RECTANGLE const &src_rect,
                               Surface &dest,RECTANGLE const &dest_rect,EFFECTS const *effects,void *extra);

        // clipping and cropping
        int ClipAndCrop(Surface const &src,Surface const &dest,RECTANGLE &rect);
        int ClipAndCrop(Surface const &src,RECTANGLE &src_rect,Surface const &dest,RECTANGLE &dest_rect);

        // bound surface management
        void FreeSurfaces();
        void InvalidateSurfaces();

        // bound surface list
        List<Surface> SurfaceList;

        // format converter
        CONVERTER Converter;

        // internal software surface
        class SURFACE : public Interface::SURFACE
        {
            public:
            
                // setup
                SURFACE();
                ~SURFACE();

                // initialize
                int Init(int &width,int &height,FORMAT &format,int &type,int &layout,int &advance);

                // surface interface
                virtual void* Lock(int wait);
                virtual void Unlock();
                virtual int LockCount();
                virtual int NativeType();
                virtual void* GetNative();

            private:

                // data
                void *Buffer;
                int Count;
        };

        // internal surface management
        virtual Interface::SURFACE* RequestSurface(int &width,int &height,FORMAT &format,int &type,int &layout,int &advance);

        // object attach and detach
        virtual int Attach(Surface *surface);
        virtual int Detach(Surface *surface);
};









#endif
